
# q-expansion of newform 2520.2.bi.c, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2520
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[631, [1, 0]], [1261, [1, 0]], [281, [1, 0]], [2017, [1, 0]], [1081, [0, -1]]]
aps_data = [[0, 0], [0, 0], [0, -1], [2, -3], [-2, 2], [1, 0], [0, 0], [0, -3], [0, 0], [0, 0], [9, -9], [0, -3], [-2, 0], [3, 0], [0, -6], [0, 0], [-4, 4], [0, -2], [-5, 5], [-14, 0], [-1, 1], [0, -9], [6, 0], [0, -4], [-14, 0], [-2, 2], [0, -3], [0, -16], [-13, 13], [-6, 0], [11, 0], [0, -18], [8, -8], [-9, 0], [0, -20], [16, -16], [-10, 10], [0, 12], [18, 0], [0, 4], [10, -10], [25, 0], [0, -22], [-23, 23], [0, 0], [24, -24], [4, 0], [-16, 0], [26, -26], [0, 23], [0, -22], [22, 0], [18, -18], [16, 0], [0, 18], [-12, 12], [-2, 2], [0, -16], [-5, 5], [-8, 0], [-11, 11], [-8, 0], [-15, 0], [-14, 14], [0, 13], [0, 12], [0, -13], [1, 0], [-4, 4], [26, 0], [-30, 30], [0, 0], [35, -35], [0, 33], [1, 0], [0, 24], [-14, 14], [0, -7], [0, -24], [19, -19], [30, 0], [5, 0], [-34, 34], [3, 0], [0, -8], [0, 0], [6, 0], [0, -17], [24, 0], [-19, 0], [0, -18], [-24, 24], [-5, 5], [12, 0], [0, 1], [22, 0], [0, 30], [12, -12], [0, -1], [0, -33], [44, 0], [6, -6], [-2, 2], [0, -38], [-15, 15], [13, -13], [28, 0], [0, 38], [-12, 12], [23, 0], [0, 43], [-26, 26], [-34, 0], [31, -31], [24, 0], [40, -40], [11, 0], [18, -18], [0, 6], [40, 0], [-35, 35], [-13, 0], [0, 0], [-44, 44], [0, -17], [48, 0], [0, 26], [0, 2], [17, 0], [0, 31], [-11, 11], [30, 0], [0, -19], [26, 0], [0, 8], [-25, 0], [34, -34], [-8, 8], [-32, 0], [10, -10], [-40, 0], [0, 2], [-32, 32], [-50, 0], [11, -11], [-40, 0], [43, 0], [52, -52], [0, 16], [0, -34], [0, -2], [52, 0], [29, 0], [0, -2], [3, -3], [-8, 0], [0, -25], [0, -18], [-5, 0], [52, -52], [0, 38], [16, 0], [49, 0], [0, 16], [-6, 6], [36, -36], [-23, 23], [-11, 11], [-35, 0], [0, -12], [-24, 24], [46, 0], [0, -48], [15, -15], [5, -5], [0, 0], [-39, 0], [30, -30], [-24, 0], [0, -35], [0, -31], [12, 0], [-15, 0], [0, -24], [-10, 10], [-4, 4], [34, -34], [7, -7], [0, 23], [-6, 6], [0, -21], [-6, 0], [0, -32], [0, -8], [-52, 52], [-60, 60], [37, -37], [0, -46], [-2, 0], [0, 20], [-66, 66], [17, 0], [0, 50], [14, -14], [-54, 0], [42, -42], [0, -8], [0, -14], [-28, 0], [-60, 60], [0, -43], [-20, 0], [-12, 0], [0, 0], [36, -36], [0, -15], [-40, 40], [-20, 20], [0, 32], [-6, 0], [0, -25], [67, 0], [0, -30], [0, 0], [50, 0], [-6, 0], [0, -32], [30, -30], [0, -13], [0, -10], [51, -51], [36, -36], [-63, 0], [-6, 6], [-1, 0], [38, -38], [0, -29], [0, 2], [20, 0], [-30, 0], [-6, 6], [0, -43], [-23, 23], [0, 18], [-24, 0], [0, 16], [-7, 0], [6, -6], [-32, 32], [48, 0], [-11, 0], [0, 4], [22, -22], [5, 0], [0, 0], [0, 12], [11, -11], [-25, 25], [-42, 0], [0, 66], [-57, 57], [72, 0], [-57, 57], [19, 0], [-18, 18], [0, -11], [74, 0], [-22, 0], [-36, 0], [74, -74], [0, 57], [-68, 68], [71, -71], [0, 23], [-38, 0], [0, -52], [0, 54], [45, -45], [0, 13], [0, 4], [80, -80], [23, -23], [58, 0], [35, 0], [0, -60], [0, -66], [-77, 77], [-6, 0], [21, -21], [78, 0], [-12, 12], [52, -52], [0, 54], [-32, 0], [-61, 0], [-42, 42], [0, 41], [-4, 0], [0, -6], [-39, 0], [0, 26], [0, 26], [-68, 68]]
